/*
              Long time timer ver.3 OK, updated for DS3231 
              Attiny85 I2C, RTC, sleep, PCI alarm
              Consumption in sleep 0,4mA (3V) 
              RTC initiate alarm each 10-th days (line 115)
              Kamil Zachej, october 2023

  Ain0 (D 5) PB5  1|    |8  Vcc
  Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
  Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
             GND  4|    |5  PB0 (D 0) pwm0             
*/

#include <avr/sleep.h>                  //Sleep Modes
#include <avr/power.h>                  //Power management
#include <Time.h>                       //Library for Time OK
#include <TinyWireM.h>                  //Library of Wire to support DS1307RTC (Real-Time Clock) - Pins to ATTINY85: #5 (SDA), #7 (SCK/SCL)
#include <DS3232RTC.h>                  
#define DS3232_I2C_ADDRESS 0x57         

constexpr uint8_t RTC_INT_PIN {1};      //RTC provides an alarm signal on this pin PB1
const byte alarmLed = 3;                //attiny pin PB3, pin2
const byte alarmReset = 4;              //attiny pin PB4, pin3
uint8_t alarmDay;
uint8_t actMonth;
uint16_t actYear;
uint8_t numberDays;
uint8_t dayMonth; 

DS3232RTC myRTC;
ISR (PCINT0_vect) {}                     // do something interesting here, if interrupt is called

void setup() {
  pinMode(RTC_INT_PIN, INPUT_PULLUP);
  pinMode(alarmLed, OUTPUT);
  pinMode(alarmReset, INPUT_PULLUP);
  
  // pin change interrupt (PB1)
  PCMSK  |= bit (PCINT1);               // want pin PB1 / pin 6 selected
  GIFR   |= bit (PCIF);                 // clear any outstanding interrupts
  GIMSK  |= bit (PCIE);                 // enable pin change interrupts 

  myRTC.begin();
  setSyncProvider(myRTC.get);           // Update the time with data of RTC (Real Time Clock)
  TinyWireM.begin();                    // initialize I2C lib

  // initialize the alarms to known values, clear the alarm flags, clear the alarm interrupt flags
  myRTC.setAlarm(DS3232RTC::ALM1_MATCH_DATE, 0, 0, 0, 1);
  myRTC.setAlarm(DS3232RTC::ALM2_MATCH_DATE, 0, 0, 0, 1);
  myRTC.alarm(DS3232RTC::ALARM_1);
  myRTC.alarm(DS3232RTC::ALARM_2);
  myRTC.alarmInterrupt(DS3232RTC::ALARM_1, false);
  myRTC.alarmInterrupt(DS3232RTC::ALARM_2, false);
  myRTC.squareWave(DS3232RTC::SQWAVE_NONE);

   // Read datum of Clock Mode (Decimal or Binary)stored in RTC
  TinyWireM.beginTransmission(DS3232_I2C_ADDRESS);               // Open I2C line in write mode
  TinyWireM.write((byte)0x08);                                   // Set the register pointer to (0x08)
  TinyWireM.endTransmission();                                   // End Write Transmission
  TinyWireM.requestFrom(DS3232_I2C_ADDRESS, 1);                  // In this case read only 1 byte

   // Write data of Clock Mode (Decimal or Binary) in RTC
  TinyWireM.beginTransmission(DS3232_I2C_ADDRESS);               // Open I2C line in write mode
  TinyWireM.write((byte)0x08);                                   // Set the register pointer to (0x08)
  TinyWireM.endTransmission();                                   // End Write Transmission
    
   //clear the alarm flags            
  myRTC.alarm(DS3232RTC::ALARM_1);
  myRTC.alarm(DS3232RTC::ALARM_2);
  // configure the INT/SQW pin for "interrupt" operation (i.e. disable square wave output)
  myRTC.squareWave(DS3232RTC::SQWAVE_NONE);

  //enable interrupt output for Alarm 2 only
  myRTC.alarmInterrupt(DS3232RTC::ALARM_1, false);
  myRTC.alarmInterrupt(DS3232RTC::ALARM_2, true);

  //settingTime ();    
  //setTime(16, 54, 0, 11, 7, 2023);          // Uncomment - Initial time & date setup of RTC
  //RTC.set(now());                           // Update the RTC with current time
}

void loop() {
    dayMonth = day();
    actMonth = month();
    actYear = year(); 
    numberDays = numberOfDaysInMonth(actYear, actMonth);
      // check to see if the INT/SQW pin is low, i.e. an alarm has occurred
    byte alarmState = digitalRead(alarmReset);
      // reset the alarm flag
    myRTC.alarm(DS3232RTC::ALARM_2);   
    digitalWrite(alarmLed, HIGH);
    delay(100);
    digitalWrite(alarmLed, LOW);
    delay(3000);
    if (alarmState == LOW) {
          alarmAdjust();
          goToSleep();            
        }                                                                             
}

void goToSleep ()  {
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);  // PC interrupt??  set Reset interrupt
  ADCSRA = 0;                           // turn off ADC
  power_all_disable ();                 // power off ADC, Timer 0 and 1, serial interface
  sleep_enable();
  sleep_cpu(); 
                                        // sleeping (waiting for PCINT1)
  sleep_disable();   
  power_all_enable();                   // power everything back on
  }                                     // end of goToSleep 

  // Calculation for next alarm day
void alarmAdjust ()
{     
  alarmDay = (dayMonth + 10);
    if (alarmDay > numberDays)
        { 
        alarmDay = alarmDay-numberDays;
          }    
  myRTC.setAlarm(DS3232RTC::ALM2_MATCH_DATE, 10, 19, alarmDay); 
   }

    //calculation number of days for current Month        
uint8_t numberOfDaysInMonth(uint16_t year, uint8_t month)           //calculation number of days
{  
	if ( month == 2 )  
		return ( (year % 400 == 0) || (year % 4 == 0 && year % 100 != 0) ) ? 29 : 28;		
	else if ( month == 4 || month == 6 || month == 9 || month == 11 )  
  		return 30;  		
	return 31;
}
